package de.javamagazin.store;

import java.io.IOException;
import java.io.StringReader;
import java.net.URL;
import java.security.Key;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.Collections;

import javax.xml.transform.stream.StreamSource;

import org.apache.axis.AxisFault;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.message.SOAPHeaderElement;
import org.apache.axis.utils.XMLUtils;
import org.w3c.dom.Document;
import org.xmltrustcenter.verifier.SimpleTrustVerifier;

import com.verisign.messaging.MessageValidity;
import com.verisign.messaging.WSSecurity;

/**
 * @axis.handler name="ServerSecurity"
 * @axis.parameter name="keystore" value="keystore_server"
 */
public class ServerSecurityHandler extends AbstractSecurityHandler {

	public static final String WSSE_URI = "http://schemas.xmlsoap.org/ws/2002/07/secext";
	public static final String SECURITY = "Security";

	/**
	 * inits the keystore
	 */
	public void init() {
		log.info("ServerSecurityHandler: Initialiazing...");
		
		URL keyStoreUrl = getClass().getResource((String)getOption("keystore"));
		
		registerSecurityProvider();
		
		try {
			loadKeyStore(keyStoreUrl.openStream(), "storePwd".toCharArray());
		} catch (IOException e) {
			log.info("Error loading keystore " + e.getMessage());
		}
	}
	
	/** 
	 * @see org.apache.axis.handlers.BasicHandler#invoke(org.apache.axis.MessageContext)
	 */
	public void invoke(MessageContext msgCtx) throws AxisFault {
		if (keyStore == null)
			throw new AxisFault("ServerSecurityHandler:key store not initialized ");try {
				Message msg = msgCtx.getRequestMessage();
				Document doc = msg.getSOAPEnvelope().getAsDocument();

				// uncomment to see encrypted SOAP message
				//String encStr = XMLUtils.DocumentToString(doc);
				//log.info(encStr);

				verifyAndDecrypt(doc);

				String str = XMLUtils.DocumentToString(doc);
				// uncomment to see SOAP message
				log.info(str);

				StreamSource s = new StreamSource(new StringReader(str));
				msg.getSOAPPart().setContent(s);

				// mark Security header as processed
				SOAPHeaderElement secHeader = msg.getSOAPEnvelope().getHeaderByName(WSSE_URI, SECURITY);
				if( secHeader != null ) secHeader.setProcessed(true);

			} catch (Exception e) {
				e.printStackTrace();
				throw new AxisFault("Exception in ServerSecurityHandler : " + e.getMessage());
				}
	}
	
	/**
	 * verifiziert und entschlsselt die SOAP Nachricht.
	 * @param soapMsg die Soap Nachricht als DOM-Document
	 * @throws Exception
	 */
	private void verifyAndDecrypt(Document soapMsg) throws Exception {
		log.info("ServerSecurityHandler: verify signature and decrypt message...");

		// get client's certificate and create a trust
		// verifier using its public key
		X509Certificate certClient = (X509Certificate)keyStore.getCertificate("Client");
		SimpleTrustVerifier trustVerifier = new SimpleTrustVerifier(Collections.singleton(certClient.getPublicKey()));

		// get server's private key for decrypting the
		// symmetric data encryption key
		Key keyDecKey = (PrivateKey)keyStore.getKey("Server", "keyPwd".toCharArray());
		// start...
		WSSecurity wsse = new WSSecurity();
		MessageValidity[] validities = wsse.verifyAndDecrypt(soapMsg, trustVerifier, null, keyDecKey, null);
		// check if signature was valid
		for (int i = 0; i < validities.length; i++) {
			if (!validities[i].isValid()) {
				log.warn("ServerSecurityHandler: Verification failure : " + validities[i].getReasonInvalid());
					
				throw new AxisFault("ServerSecurityHandler: Verification failure ");
					
				}
		}
		log.info("ServerSecurityHandler: done.");
	}
}